"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bubbleEvents = [
    'submit'
];
function isEventTag(tag) {
    return (tag.taggingMethod === 'event' ||
        tag.taggingMethod === undefined);
}
function isIdentifyFormTag(tag) {
    return (tag.formAction === 'identify' &&
        tag.taggingMethod === 'form');
}
function isTrackFormTag(tag) {
    return (tag.formAction === 'track' &&
        tag.taggingMethod === 'form');
}
const getBucketPath = (workspaceId, sourceId) => `https://s3-us-west-2.amazonaws.com/onboarding-service-instrumentation-specs-production/instrumentation-spec-${workspaceId}-${sourceId}.json`;
class Tracktor {
    constructor(workspaceId, sourceId, documentNode) {
        this.workspaceId = workspaceId;
        this.sourceId = sourceId;
        this.documentNode = documentNode;
        this.load = (specs) => {
            this.setListenerMap(this.toSpecMapByHandlerKey(specs));
            this.initialized = true;
            this.installListeners();
        };
        this.getInstrumentationSpec = () => {
            return fetch(getBucketPath(this.workspaceId, this.sourceId), {
                headers: {
                    'Cache-Control': 'no-cache'
                }
            }).then(res => res.ok ? res.json() : []);
        };
        this.setListenerMap = (eventToTagMap) => {
            this.listenerMap = Object.entries(eventToTagMap).reduce((listenerMap, [event, tags]) => {
                return Object.assign(Object.assign({}, listenerMap), { [event]: e => {
                        const tag = tags.find(s => this.isMatch(s, e.target));
                        if (!tag) {
                            return;
                        }
                        if (isEventTag(tag)) {
                            this.handleEventTag(tag);
                            return;
                        }
                        if (isIdentifyFormTag(tag)) {
                            this.handleIdentifyFormTag(tag);
                            return;
                        }
                        if (isTrackFormTag(tag)) {
                            this.handleTrackFormTag(tag);
                            return;
                        }
                    } });
            }, {});
        };
        if (!this.workspaceId || !this.sourceId) {
            throw new Error('workspaceId and sourceId must be specified');
        }
        if (Tracktor.instance) {
            return Tracktor.instance;
        }
        Tracktor.instance = this;
    }
    start() {
        return Promise.resolve()
            .then(this.getInstrumentationSpec)
            .then(this.load);
    }
    restart(specs) {
        this.removeListeners();
        this.load(specs);
    }
    isInitialized() {
        return this.initialized;
    }
    toSpecMapByHandlerKey(specs) {
        return specs.reduce((res, spec) => (Object.assign(Object.assign({}, res), { [spec.handler]: [...(res[spec.handler] || []), spec] })), {});
    }
    isMatch(tag, childElement, maxAncestorHeight = 3) {
        let curEl = childElement;
        let curHeight = 0;
        while (curEl && curHeight < maxAncestorHeight) {
            if (curEl.matches(tag.selector)) {
                return true;
            }
            curEl = curEl.parentElement;
            curHeight++;
        }
        return false;
    }
    handleEventTag(tag) {
        window.analytics.track(tag.event, {}, {
            context: {
                visual_tagger: true
            }
        });
    }
    handleIdentifyFormTag(tag) {
        const identifyBindings = tag.bindings;
        const userIdentifier = identifyBindings.find(b => b.userIdentifier);
        const bindings = identifyBindings.filter(b => !b.userIdentifier);
        if (!userIdentifier) {
            console.error(`A unique identifier must be provided as a property binding for the form tagged at ${tag.selector} `);
            return;
        }
        const formEl = this.documentNode.querySelector(tag.selector);
        if (!formEl) {
            console.error(`form tagged at ${tag.selector} cannot be found in the current page `);
            return;
        }
        const identifierInput = formEl.querySelector(userIdentifier.selector);
        if (!identifierInput) {
            console.error(`Identifier input tagged at ${userIdentifier.selector} cannot be found in the current form`);
            return;
        }
        const traits = bindings.reduce((traits, binding) => {
            const fieldValue = formEl.querySelector(binding.selector);
            return Object.assign(Object.assign({}, traits), { [binding.field]: fieldValue && fieldValue.value });
        }, {});
        window.analytics.identify(identifierInput.value, traits, {
            context: {
                visual_tagger: true
            }
        });
    }
    handleTrackFormTag(tag) {
        const bindings = tag.bindings;
        const formEl = this.documentNode.querySelector(tag.selector);
        if (!formEl) {
            console.error(`form tagged at ${tag.selector} cannot be found in the current page `);
            return;
        }
        const properties = bindings.reduce((props, binding) => {
            const fieldValue = formEl.querySelector(binding.selector);
            return Object.assign(Object.assign({}, props), { [binding.field]: fieldValue && fieldValue.value });
        }, {});
        window.analytics.track(tag.event, properties, {
            context: {
                visual_tagger: true
            }
        });
    }
    installListeners() {
        Object.entries(this.listenerMap).forEach(([event, listener]) => {
            const useCapture = !bubbleEvents.includes(event);
            this.documentNode.addEventListener(event, listener, useCapture);
        });
    }
    removeListeners() {
        Object.entries(this.listenerMap).forEach(([event, listener]) => {
            const useCapture = !bubbleEvents.includes(event);
            this.documentNode.removeEventListener(event, listener, useCapture);
        });
    }
}
exports.default = Tracktor;
//# sourceMappingURL=tracktor.js.map